@@ -100,8 +100,8 @@ typedef unsigned long useconds_t;
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v2.4.12"
-#define CUPS_MINIMAL "CUPS/2.4.12"
+#define CUPS_SVERSION "CUPS v2.4.15"
+#define CUPS_MINIMAL "CUPS/2.4.15"
 
 
 /*
@@ -169,6 +169,13 @@ typedef unsigned long useconds_t;
 #define CUPS_DEFAULT_ERROR_POLICY "stop-printer"
 
 
+/*
+ * Default PeerCred value...
+ */
+
+#define CUPS_DEFAULT_PEER_CRED "on"
+
+
 /*
  * Default MaxCopies value...
  */
@@ -751,13 +758,6 @@ typedef unsigned long useconds_t;
 /* #undef HAVE_SYS_VFS_H */
 
 
-/*
- * Location of macOS localization bundle, if any.
- */
-
-/* #undef CUPS_BUNDLEDIR */
-
-
 /*
  * Do we have XPC?
  */
