@@ -19,8 +19,8 @@
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v2.4.12"
-#define CUPS_MINIMAL "CUPS/2.4.12"
+#define CUPS_SVERSION "CUPS v2.4.15"
+#define CUPS_MINIMAL "CUPS/2.4.15"
 
 
 /*
@@ -88,6 +88,13 @@
 #define CUPS_DEFAULT_ERROR_POLICY "stop-printer"
 
 
+/*
+ * Default PeerCred value...
+ */
+
+#define CUPS_DEFAULT_PEER_CRED "on"
+
+
 /*
  * Default MaxCopies value...
  */
@@ -683,17 +690,6 @@
 /* #undef HAVE_SYS_VFS_H */
 
 
-/*
- * Location of localization bundle, if any.
- */
-
-#if TARGET_OS_OSX
-#  define CUPS_BUNDLEDIR "/System/Library/Frameworks/ApplicationServices.framework/Versions/A/Frameworks/PrintCore.framework/Versions/A"
-#else
-#  define CUPS_BUNDLEDIR "/System/Library/PrivateFrameworks/PrintKit.framework/Versions/A"
-#endif /* TARGET_OS_OSX */
-
-
 /*
  * Do we have XPC?
  */
